/*******************************************************************************
NAME                        INTERRUPTED MOLLWEIDE

PURPOSE:	Transforms input Easting and Northing to longitude and
		latitude for the Interrupted Mollweide projection.  The
		Easting and Northing must be in meters.  The longitude
		and latitude values will be returned in radians.

PROGRAMMER              DATE
----------              ----
J. Descloitres, UMD	July, 1998
(Based on GCTP routines written by D. Steinwand, S. Nelson and T. Mittan)

ALGORITHM REFERENCES

1.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.

2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
*******************************************************************************/
#include "cproj.h"

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
static double R;		/* Radius of the earth (sphere) */
static double false_easting;	/* x offset in meters			*/
static double false_northing;	/* y offset in meters			*/
static double lon_center[6];	/* Central meridians, one for each region */
static double feast[6];		/* False easting, one for each region */

/* Initialize the Interrupted Mollweide projection
  --------------------------------------------*/
imolwinvint(r, false_east, false_north) 
double r; 			/* (I) Radius of the earth (sphere) */
double false_east;		/* x offset in meters                   */
double false_north;		/* y offset in meters                   */
{
/* Place parameters in static storage for common use
  -------------------------------------------------*/
false_easting = false_east;
false_northing = false_north;
R = r;

/* Initialize central meridians for each of the 6 regions
  -------------------------------------------------------*/
lon_center[0] = -100 * D2R;	/* -100.0 degrees */
lon_center[1] =   30 * D2R;	/*   30.0 degrees */
lon_center[2] =  -80 * D2R;	/*  -80.0 degrees */
lon_center[3] =  -80 * D2R;	/*  -80.0 degrees */
lon_center[4] =   20 * D2R;	/*   20.0 degrees */
lon_center[5] =  140 * D2R;	/*  140.0 degrees */

/* Initialize false eastings for each of the 6 regions
  ----------------------------------------------------*/
feast[0] = 0.900316316158 * R * -100 * D2R;
feast[1] = 0.900316316158 * R *   30 * D2R;
feast[2] = 0.900316316158 * R *  -80 * D2R;
feast[3] = 0.900316316158 * R *  -80 * D2R;
feast[4] = 0.900316316158 * R *   20 * D2R;
feast[5] = 0.900316316158 * R *  140 * D2R;

/* Report parameters to the user
  -----------------------------*/
ptitle("INTERRUPTED MOLLWEIDE EQUAL-AREA"); 
radius(r);
offsetp(false_easting,false_northing);
return(OK);
}

/* Interrupted Mollweide inverse equations--mapping x,y to lat,long 
  -------------------------------------------------------------*/
imolwinv(x, y, lon, lat)
double x;		/* (I) X projection coordinate */
double y;		/* (I) Y projection coordinate */
double *lon;		/* (O) Longitude */
double *lat;		/* (O) Latitude */
{
double adjust_lon();    /* Function to adjust longitude to -180 - 180 */
double arg;
double theta;
double temp;
long region;

/* Inverse equations
  -----------------*/
x -= false_easting;
y -= false_northing;
if (y >= 0.0)
   {
   if (x <= 0.900316316158 * R * -0.698131700798) region = 0;		/* If to the left of -40 */
   else region = 1;
   }
else
   {
   if (x <= 0.900316316158 * R * -1.74532925199) region = 2;		/* If between -180 and -100 */
   else if (x <= 0.900316316158 * R * -0.349065850399) region = 3;	/* If between -100 and -20 */
   else if (x <= 0.900316316158 * R * 1.3962634016) region = 4;		/* If between -20 and 80 */
   else region = 5;							/* If between 80 and 180 */
   }
x = x - feast[region];

   arg = y / (1.4142135623731 * R);
   if (fabs(arg) > 1.0) return(IN_BREAK);
   theta = asin(arg);
   *lon = lon_center[region]+(x/(0.900316316158 * R * cos(theta)));
   if(*lon < -(PI + EPSLN)) return(IN_BREAK);
   arg = (2.0 * theta + sin(2.0 * theta)) / PI;
   if (fabs(arg) > 1.0) return(IN_BREAK);
   *lat = asin(arg);
   if (region == 0 &&
       *lon <= -2.94925737002 &&			/* West of -168.98 deg in Bering Strait */
       *lat - 1.047197551197 >= (*lon + PI) / 2.)	/* North of Saint Lawrence and Saint Matthew Islands */
     return(IN_BREAK);					/* Erase this region from region 0 */
   if (region == 1 &&
       *lon >= PI && *lon <= TWO_PI - 2.94925737002 &&	/* East of 180 deg in Siberia and West of -168.98 deg in Bering Strait */
       *lat - 1.047197551197 >= (*lon - PI) / 2.) {	/* North of Saint Lawrence and Saint Matthew Islands */
     *lon -= TWO_PI;					/* Adjust longitude to respect the valid range */
     return(OK);					/* Add this region to region 1 */
   }
   if (*lat >= 1.047197551197 &&				/* North of 60 deg */
       *lon <= -0.174532925199 && *lon >= -0.698131700798 &&	/* West of -10 deg and East of -40 deg */
       *lat - 1.0471975512 >= (*lon + 0.698131700798) / 2.) {	/* North of Iceland in Denmark Strait */
     if (region == 1) return(IN_BREAK);				/* Erase this region from region 1 */
                 else return(OK);				/* Add this region to region 0 */
   }
/* because of precision problems, long values of 180 deg and -180 deg
   may be mixed.
   ----------------------------------------------------------------*/
if (((x < 0) && (PI - *lon < EPSLN)) || ((x > 0) && (PI + *lon < EPSLN)))
   *lon = -(*lon);

/* Are we in a interrupted area?  If so, return status code of IN_BREAK.
  ---------------------------------------------------------------------*/
if (region == 0 && (*lon < -(PI + EPSLN) || *lon > -0.698131700798))
							return(IN_BREAK);
if (region == 1 && (*lon < -0.698131700798 || *lon > PI + EPSLN))
							return(IN_BREAK);
if (region == 2 && *lon < -(PI + EPSLN))   return(IN_BREAK);
if (region == 3 && *lon > -0.349065850399) return(IN_BREAK);
if (region == 4 && (*lon < -0.349065850399 || *lon > 1.3962634016))
							return(IN_BREAK);
if (region == 5 && (*lon < 1.3962634016 || *lon > PI + EPSLN))
							return(IN_BREAK);
return(OK);
}

