/*******************************************************************************
NAME                        INTERRUPTED SINUSOIDAL

PURPOSE:	Transforms input longitude and latitude to Easting and
		Northing for the Interrupted sinusoidal projection.  The
		longitude and latitude must be in degrees.  The Easting
		and Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
J. Descloitres, UMD	July, 1998
(Based on GCTP routines written by D. Steinwand, S. Nelson and T. Mittan)

ALGORITHM REFERENCES

1.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.

2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
*******************************************************************************/
#include "cproj.h"

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
static double R;		/* Radius of the earth (sphere) */
static double false_easting;	/* x offset in meters			*/
static double false_northing;	/* y offset in meters			*/
static double lon_center[5];	/* Central meridians, one for each region */
static double feast[5];		/* False easting, one for each region */




/* Initialize the USGS interrupted sinusoidal projection
  --------------------------------------------*/
int intersinint1(r, false_east, false_north) 
double r; 			/* (I) Radius of the earth (sphere) */
double false_east;		/* x offset in meters                   */
double false_north;		/* y offset in meters                   */
{
/* Place parameters in static storage for common use
  -------------------------------------------------*/
false_easting = false_east;
false_northing = false_north;
R = r;

/* Initialize central meridians for each of the 3 regions
  -------------------------------------------------------*/
lon_center[0] =  -90;	/*  -90.0 degrees */
lon_center[1] =   20;	/*   20.0 degrees */
lon_center[2] =  130;	/*  130.0 degrees */

/* Initialize false eastings for each of the 3 regions
  ----------------------------------------------------*/
feast[0] = R * -110;
feast[1] = 0;
feast[2] = R *  110;

/* Report parameters to the user
  -----------------------------*/
ptitle("USGS INTERRUPTED SINUSOIDAL EQUAL-AREA"); 
radius(r);
offsetp(false_easting,false_northing);
return(OK);
}




/* Initialize the interrupted sinusoidal variant projection
  --------------------------------------------*/
int intersinint2(r, false_east, false_north) 
double r; 			/* (I) Radius of the earth (sphere) */
double false_east;		/* x offset in meters                   */
double false_north;		/* y offset in meters                   */
{
/* Place parameters in static storage for common use
  -------------------------------------------------*/
false_easting = false_east;
false_northing = false_north;
R = r;

/* Initialize central meridians for each of the 5 regions
  -------------------------------------------------------*/
lon_center[0] =  -90;	/*  -90.0 degrees */
lon_center[1] =  -60;	/*  -60.0 degrees */
lon_center[2] =   40;	/*   40.0 degrees */
lon_center[3] =   20;	/*   20.0 degrees */
lon_center[4] =  130;	/*  130.0 degrees */

/* Initialize false eastings for each of the 5 regions
  ----------------------------------------------------*/
feast[0] = R * (-110 + 9.5);
feast[1] = R * ( -80 + 9.5);
feast[2] = R * (  20 + 9.5);
feast[3] = R * (   0 + 9.5);
feast[4] = R * ( 110 + 9.5);

/* Report parameters to the user
  -----------------------------*/
ptitle("INTERRUPTED SINUSOIDAL VARIANT EQUAL-AREA"); 
radius(r);
offsetp(false_easting,false_northing);
return(OK);
}




/* USGS interrupted sinusoidal forward equations--mapping lat,long to x,y
  -------------------------------------------------------------*/
int intersinfor1(lon, lat, x, y)
double lon;			/* (I) Longitude */
double lat;			/* (I) Latitude */
double *x;			/* (O) X projection coordinate */
double *y;			/* (O) Y projection coordinate */
{
double adjust_lon();	/* Function to adjust longitude to -180 - 180 */
double delta_lon;	/* Delta longitude (Given longitude - center */
long region;

/* Forward equations
  -----------------*/
 if (fabs(lon) > 180) return(ERROR);
 if      (fabs(lon + 90) <= 70) region = 0;	/* Lon. range [-160, -20] */
 else if (fabs(lon - 20) <= 40) region = 1;	/* Lon. range [ -20, +60] */
 else                           region = 2;	/* Longitude range [ +60,+180] and [-180,-160] */
 delta_lon = adjust_lon(lon - lon_center[region]);
      if (delta_lon < -180) delta_lon += 360;
 else if (delta_lon >  180) delta_lon -= 360;
 *x = false_easting  + feast[region] + R * delta_lon * cos(lat * D2R);
 *y = false_northing + R * lat;

 return(OK);
}




/* Interrupted sinusoidal variant forward equations--mapping lat,long to x,y
  -------------------------------------------------------------*/
int intersinfor2(lon, lat, x, y)
double lon;			/* (I) Longitude */
double lat;			/* (I) Latitude */
double *x;			/* (O) X projection coordinate */
double *y;			/* (O) Y projection coordinate */
{
double adjust_lon();	/* Function to adjust longitude to -180 - 180 */
double delta_lon;	/* Delta longitude (Given longitude - center */
long region;

/* Forward equations
  -----------------*/
 if (fabs(lon) > 180) return(ERROR);
 if      (lon >= -169.5 && lon <= -26)
   if (lat >= 0) region = 0;
            else region = 1;
 else if (lon >  -26   && lon <= +65)
   if (lat >= 0) region = 2;
            else region = 3;
 else
   if (lat >= 0) region = 2;
            else region = 4;
 delta_lon = adjust_lon(lon - lon_center[region]);
      if (delta_lon < -180) delta_lon += 360;
 else if (delta_lon >  180) delta_lon -= 360;
 *x = false_easting  + feast[region] + R * delta_lon * cos(lat * D2R);
 *y = false_northing + R * lat;

 return(OK);
}




/* USGS interrupted sinusoidal inverse equations--mapping x,y to lat,long 
  -------------------------------------------------------------*/
int intersininv1(x, y, lon, lat)
double x;               /* (I) X projection coordinate */
double y;               /* (I) Y projection coordinate */
double *lon;            /* (O) Longitude */
double *lat;            /* (O) Latitude */
{
double adjust_lon();    /* Function to adjust longitude to -180 - 180 */
long region;

/* Inverse equations
  -----------------*/
  x -= false_easting;
  y -= false_northing;
  *lat = y;
  if (fabs(*lat) >= 90) return(ERROR);
  if      (x <  -40)            region = 0;        /* Longitude range [-160, -20] */
  else if (x >= -40 && x <= 40) region = 1;        /* Longitude range [ -20, +60] */
  else                          region = 2;        /* Longitude range [ +60,+180] and [-180,-160] */
  *lon = (x - feast[region]) / cos(*lat * D2R);
  if (region == 1  &&  fabs(*lon) > 40) return(IN_BREAK);         /* Pixel is in projection break */
  if (region != 1  &&  fabs(*lon) > 70) return(IN_BREAK);
  *lon += lon_center[region];
       if (*lon < -180) *lon += 360;
  else if (*lon >  180) *lon -= 360;

  return(OK);
}




/* Interrupted sinusoidal variant inverse equations--mapping x,y to lat,long 
  -------------------------------------------------------------*/
int intersininv2(x, y, lon, lat)
double x;               /* (I) X projection coordinate */
double y;               /* (I) Y projection coordinate */
double *lon;            /* (O) Longitude */
double *lat;            /* (O) Latitude */
{
double adjust_lon();    /* Function to adjust longitude to -180 - 180 */
long region;

/* Inverse equations
  -----------------*/
  x -= false_easting;
  y -= false_northing;
  *lat = y;
  if (fabs(*lat) >= 90) return(ERROR);
  if (x <= -46+9.5)
    if (*lat >= 0) region = 0;
              else region = 1;
  else if (x > -46+9.5 && x <= 45+9.5)
    if (*lat >= 0) region = 2;
              else region = 3;
  else
    if (*lat >= 0) region = 2;
              else region = 4;
  *lon = lon_center[region] + (x - feast[region]) / cos(*lat * D2R);
  switch (region) {		/* Pixel is in projection break */
    case 0: if (*lon < -169.5 || *lon >  -26)   return(IN_BREAK);  break;
    case 1: if (*lon < -169.5 || *lon >  -26)   return(IN_BREAK);  break;
    case 2: if (*lon <  -26   || *lon > +190.5) return(IN_BREAK);  break;
    case 3: if (*lon <  -26   || *lon >  +65)   return(IN_BREAK);  break;
    case 4: if (*lon <  +65   || *lon > +190.5) return(IN_BREAK);  break;
  }
       if (*lon < -180) *lon += 360;
  else if (*lon >  180) *lon -= 360;

  return(OK);
}
